/***********************************************************
*            WPManager.c - Wallpaper Manager               *
* This pops up a dialog to manage the wallpaper set        *
*                         *  *  *  *                       *
* Coding:                                                  *
*   Version 1.1 - Visigoth (Shaheen Gandhi in real life)   *
*                 e-mail: sgandhi@andrew.cmu.edu           *
*                         *  *  *  *                       *
* Last Update:  October 20 , 1999 7:30 AM                  *
*                         *  *  *  *                       *
* Copyright (c) 1999 Shaheen Gandhi                        *
***********************************************************/

#define NO_INCLUDE_WHARFDATA
#include <stdio.h>
#include "resource.h"
#include "lswallpaper.h"

extern HWND GetLitestepWnd();
extern BOOL ParseBangCommand(HWND caller, char *command, char *args);

HWND hDlg = NULL;
struct WallpaperRecordList *wprl;
BOOL changed = FALSE, modal = FALSE, saved = FALSE;

BOOL SaveFile(HWND hDlg)
{
  FILE *fpList = NULL;
  HWND text = GetDlgItem(hDlg, IDC_FULLPATH);
  TCHAR output[_MAX_PATH + 20], tiledStr[17];

  if(!wprl) {
    if(MessageBox(hDlg, "You haven't specified any wallpapers.  Are you sure you want to save?", "LSWallpaper", MB_YESNO | MB_ICONQUESTION) == IDNO) {
      return FALSE;
    }
  }

  fpList = fopen(settings.WallpaperListPath, "w");

  if(fpList) {
    WPRLMovetoHead(&wprl);

    if(wprl) {
      while(wprl->next) {
        sprintf(tiledStr, "\t%d\n", wprl->wpr->tiled);
        strcpy(output, wprl->wpr->path);
        strcat(output, tiledStr);
        fputs(output, fpList);
        WPRLMoveNext(&wprl, 1);
      }

      sprintf(tiledStr, "\t%d\n", wprl->wpr->tiled);
      strcpy(output, wprl->wpr->path);
      strcat(output, tiledStr);
      fputs(output, fpList);
      WPRLMoveNext(&wprl, 1);
    }

    fclose(fpList);
    changed = FALSE;
    saved = TRUE;
    sprintf(output, "%s saved!", settings.WallpaperListPath);

    SendMessage(text, WM_SETTEXT, 0, (LPARAM)output);
    return TRUE;
  } else
    return FALSE;
}

void AddWallpaper(HWND hDlg, char *path)
{
  HWND list = GetDlgItem(hDlg, IDC_WALLPAPERLIST);
  HWND tile = GetDlgItem(hDlg, IDC_TILED);
  int count = SendMessage(list, LB_GETCOUNT, 0, 0), compare;
  struct WallpaperRecord *wpr = (struct WallpaperRecord *)malloc(sizeof(struct WallpaperRecord));
  char drive[_MAX_DRIVE + _MAX_DIR + 2], dir[_MAX_DIR], *pointer = settings.BitmapPath + strlen(settings.BitmapPath) - 1;
  wpr->tiled = FALSE;

  _splitpath(path, drive, dir, NULL, NULL);
  strcat(drive, dir);

  compare = _strnicmp(drive, settings.BitmapPath, strlen(settings.BitmapPath));

  wpr->path = compare ? (char *)malloc(strlen(path) + 1) : (char *)malloc(strlen(settings.BitmapPath) + strlen(path) + 2);
  wpr->tiled = FALSE;

  if(compare)
    strcpy(wpr->path, path);
  else {
    strcpy(wpr->path, settings.BitmapPath);
    if(!(strrchr(settings.BitmapPath, '\\') == pointer))
      strcat(wpr->path, "\\");
    pointer = strrchr(path, '\\') + 1;
    strcat(wpr->path, pointer);
  }

  wprl = WPRLAdd(&wprl, wpr);
  count = SendMessage(list, LB_INSERTSTRING, -1, (LPARAM)(compare ? path : pointer));
  SendMessage(list, LB_SETITEMDATA, count, (LPARAM)wprl);
  SendMessage(list, LB_SELITEMRANGE, FALSE, MAKELPARAM(0, count));
  SendMessage(list, LB_SELITEMRANGE, TRUE, MAKELPARAM(count, count));
  EnableWindow(tile, TRUE);
  SendMessage(tile, BM_SETCHECK, BST_UNCHECKED, 0);
  tile = GetDlgItem(hDlg, IDC_SAVE);
  EnableWindow(tile, TRUE);
  changed = TRUE;
}


void OnAdd(HWND hDlg)
{
  HWND list = GetDlgItem(hDlg, IDC_WALLPAPERLIST);
  char file[_MAX_PATH], file_total[_MAX_PATH], title[] = "Add Wallpapers", filter[] = "Bitmap Files (*.bmp)\0*.bmp\0", *p = NULL, *p2 = NULL;
  OPENFILENAME ofn;

  strcpy(file, "*.bmp\0");
  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = hDlg;
  ofn.lpstrFilter = filter;
  ofn.lpstrCustomFilter = (LPSTR) NULL;
  ofn.nMaxCustFilter = 0L;
  ofn.nFilterIndex = 0;
  ofn.lpstrFile = file;
  ofn.nMaxFile = sizeof(file);
  ofn.lpstrFileTitle = NULL; //filetitle
  ofn.nMaxFileTitle = 0; //sizeof(filetitle);
  ofn.lpstrInitialDir = NULL;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "";
  ofn.lpstrTitle = title;
  ofn.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_ALLOWMULTISELECT | OFN_HIDEREADONLY | OFN_EXPLORER; /*OFN_EXPLORER; */

  if(GetOpenFileName(&ofn)) {
    if(GetFileAttributes(ofn.lpstrFile) & FILE_ATTRIBUTE_DIRECTORY) {
      p = ofn.lpstrFile;

      strcpy(file_total, p);
      if(file_total[strlen(file_total) - 1] != '\\')
        strcat(file_total, "\\");

      p2 = file_total + strlen(file_total);

      p += strlen(p) + 1;
      while(*p) {
        strcpy(p2, p);
        AddWallpaper(hDlg, file_total);
        p += strlen(p) + 1;
      }
    } else
      AddWallpaper(hDlg, file_total);
  }
}


BOOL CALLBACK WallpaperManagerDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch(uMsg) {
  case WM_INITDIALOG:
    {
      // Initialize List box
      char file[_MAX_PATH], *p = NULL;
      HWND child;
      RECT rct;
      int i = 0, xscr = 0, yscr = 0;

      wprl = wallpapers;
      child = GetDlgItem(hWnd, IDC_WALLPAPERLIST);

      SendMessage(child, LB_RESETCONTENT, 0, 0);

      if(wprl) {
        while(wprl->next) {
          // Parse out path - get file name only.
          p = strrchr(wprl->wpr->path, '\\');
          if(p)
            strcpy(file, ++p);
          else
            strcpy(file, wprl->wpr->path);
          i = SendMessage(child, LB_ADDSTRING, 0, (LPARAM)file);
          SendMessage(child, LB_SETITEMDATA, (WPARAM)i, (LPARAM)wprl);
          WPRLMoveNext(&wprl, 1);
        }

        p = strrchr(wprl->wpr->path, '\\');
        if(p)
          strcpy(file, ++p);
        else
          strcpy(file, wprl->wpr->path);

        i = SendMessage(child, LB_ADDSTRING, 0, (LPARAM)file);
        SendMessage(child, LB_SETITEMDATA, (WPARAM)i, (LPARAM)wprl);
        WPRLMoveNext(&wprl, 1);

        p = strrchr(current->wpr->path, '\\');
        SendMessage(child, LB_SELECTSTRING, -1, (LPARAM)(LPCTSTR)(p ? ++p : current->wpr->path));
      } else {
        child = GetDlgItem(hWnd, IDC_TILED);
        EnableWindow(child, FALSE);
        child = GetDlgItem(hWnd, IDC_REMOVE);
        EnableWindow(child, FALSE);
        child = GetDlgItem(hWnd, IDC_SETWALLPAPER);
        EnableWindow(child, FALSE);
        child = GetDlgItem(hWnd, IDC_SAVE);
        EnableWindow(child, FALSE);
      }

      GetWindowRect(hWnd, &rct);
      xscr = GetSystemMetrics(SM_CXSCREEN);
      yscr = GetSystemMetrics(SM_CYSCREEN);

      SetWindowPos(hWnd, HWND_TOP, (xscr - (rct.right - rct.left))/2, (yscr - (rct.bottom - rct.top))/2, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW);

      return TRUE;
    }
    break;
  case WM_CLOSE:
    {
      if(changed) {
        int uMB = MessageBox(hWnd, "You have made changes to your wallpaper list.  Would you like to save them now?", "LSWallpaper", MB_ICONQUESTION | MB_YESNOCANCEL);

        if(uMB == IDYES)
          SaveFile(hWnd);
        else if(uMB == IDNO && !modal) {
          ParseBangCommand(hWnd, "!RESCANWALLPAPERLIST", NULL);
        } else if(uMB == IDCANCEL)
          return 0;
      }

      if(!modal) {
        DestroyWindow(hWnd);
        hDlg = NULL;
      } else {
        EndDialog(hWnd, saved);
      }
    }
    break;
  case WM_COMMAND:
    {
      WORD wNotifyCode = HIWORD(wParam), wID = LOWORD(wParam);
      HWND hWndCtl = (HWND)lParam;
      switch(wID) {
      case IDC_WALLPAPERLIST:
        switch(wNotifyCode) {
        case LBN_DBLCLK:
          {
            HWND list = GetDlgItem(hWnd, IDC_WALLPAPERLIST);
            LRESULT sel = SendMessage(list, LB_GETCURSEL, 0, 0);

            if(sel != LB_ERR) {
              current = (struct WallpaperRecordList *)SendMessage(list, LB_GETITEMDATA, sel, 0);
              UseSpecificWallpaper(&current, current->wpr->path);
            }
          }
          break;
        case LBN_SELCHANGE:
          {
            // Check / Uncheck Tiled as necessary
            HWND list = GetDlgItem(hWnd, IDC_WALLPAPERLIST);
            HWND text = GetDlgItem(hWnd, IDC_FULLPATH);
            HWND tilecheck = GetDlgItem(hWnd, IDC_TILED);
            HWND setWP = GetDlgItem(hWnd, IDC_SETWALLPAPER);
            HWND remove = GetDlgItem(hWnd, IDC_REMOVE);
            LRESULT count = SendMessage(list, LB_GETSELCOUNT, 0, 0);
            long allSame = BST_CHECKED;

            if(count != LB_ERR && count != 0) {
              struct WallpaperRecordList *wprl = NULL;
              int *sel = malloc(sizeof(int) * count);
              LRESULT retval = SendMessage(list, LB_GETSELITEMS, count, (LPARAM)sel);

              if(retval != LB_ERR) {
                if(count == 1) {
                  wprl = (struct WallpaperRecordList *)SendMessage(list, LB_GETITEMDATA, *sel, 0);
                  EnableWindow(tilecheck, TRUE);
                  SendMessage(tilecheck, BM_SETCHECK, (WPARAM)(wprl->wpr->tiled ? BST_CHECKED : BST_UNCHECKED), 0);
                  SendMessage(text, WM_SETTEXT, 0, (LPARAM)(wprl->wpr->path));
                  EnableWindow(setWP, TRUE);
                  EnableWindow(remove, TRUE);
                } else {
                  wprl = (struct WallpaperRecordList *)SendMessage(list, LB_GETITEMDATA, sel[--count], 0);
                  allSame = wprl->wpr->tiled ? BST_CHECKED : BST_UNCHECKED;
                  for(--count; count >= 0; --count) {
                    wprl = (struct WallpaperRecordList *)SendMessage(list, LB_GETITEMDATA, sel[count], 0);
                    if((wprl->wpr->tiled != FALSE) != (allSame == BST_CHECKED)) {
                      allSame = BST_INDETERMINATE;
                      break;
                    }
                  }

                  EnableWindow(tilecheck, TRUE);
                  SendMessage(tilecheck, BM_SETCHECK, (WPARAM)allSame, 0);
                  SendMessage(text, WM_SETTEXT, 0, (LPARAM)("[Multiple Items Selected]"));
                  EnableWindow(remove, TRUE);
                  EnableWindow(setWP, FALSE);
                }
              }
            } else {
              EnableWindow(tilecheck, FALSE);
              EnableWindow(setWP, FALSE);
              EnableWindow(remove, FALSE);
            }
          }
        }
        break;
      case IDC_TILED:
        if(wNotifyCode == BN_CLICKED) {
          // Set corresponding entry in wprl...
          // May have to maintain separate list for this...

          HWND tiled = GetDlgItem(hWnd, IDC_TILED);
          HWND list = GetDlgItem(hWnd, IDC_WALLPAPERLIST);
          LRESULT selcount = SendMessage(list, LB_GETSELCOUNT, 0, 0);
          if(selcount != LB_ERR && selcount != 0) {
            int *sel = malloc(sizeof(int) * selcount);
            LRESULT tileCheck = SendMessage(tiled, BM_GETCHECK, 0, 0);

            LRESULT retval = SendMessage(list, LB_GETSELITEMS, selcount, (LPARAM)sel);

            if(retval != LB_ERR) {
              for(--selcount; selcount >= 0; --selcount) {
                struct WallpaperRecordList *wprl = (struct WallpaperRecordList *)SendMessage(list, LB_GETITEMDATA, sel[selcount], 0);
                wprl->wpr->tiled = tileCheck == BST_CHECKED ? 1 : 0;
              }
              changed = TRUE;
            }
          }
        }
        break;
      case IDC_SETWALLPAPER:
        if(wNotifyCode == BN_CLICKED) {
          HWND list = GetDlgItem(hWnd, IDC_WALLPAPERLIST);
          LRESULT sel = SendMessage(list, LB_GETCURSEL, 0, 0);

          if(sel != LB_ERR) {
            current = (struct WallpaperRecordList *)SendMessage(list, LB_GETITEMDATA, sel, 0);
            UseSpecificWallpaper(&current, current->wpr->path);
          }
        }
        break;
      case IDC_ADD:
        if(wNotifyCode == BN_CLICKED) {
          // Open Files dialog... add wallpapers
          OnAdd(hWnd);
        }
        break;
      case IDC_REMOVE:
        if(wNotifyCode == BN_CLICKED) {
          // Remove current selection
          HWND list = GetDlgItem(hWnd, IDC_WALLPAPERLIST);
          int selcount = SendMessage(list, LB_GETSELCOUNT, 0, 0);
          if(selcount != LB_ERR && selcount != 0) {
            int *sel = malloc(sizeof(int) * selcount);

            LRESULT retval = SendMessage(list, LB_GETSELITEMS, selcount, (LPARAM)sel);

            if(retval != LB_ERR) {
              for(--selcount; selcount >= 0; --selcount) {
                struct WallpaperRecordList *wprl = (struct WallpaperRecordList *)SendMessage(list, LB_GETITEMDATA, sel[selcount], 0);

                if((wprl == current && MessageBox(hWnd, "The wallpaper you are about to remove from your list is the\ncurrent background.  If you proceed, the next valid wallpaper will be used.\n\nDo you wish to remove this wallpaper?", "LSWallpaper", MB_YESNO | MB_ICONQUESTION) == IDYES) || wprl != current) {
                  if(wprl == current)
                    current = (current->next != NULL) ? current->next : current->prev;

                  WPRLFree(&wprl);
                  SendMessage(list, LB_DELETESTRING, sel[selcount], 0);
                }
              }

              wallpapers = current;
              WPRLMovetoHead(&wallpapers);
              changed = TRUE;
            }
          }
        }
        break;
      case IDC_SAVE:
        if(wNotifyCode == BN_CLICKED) {
          // Write out to filev
          if(!SaveFile(hWnd))
            MessageBox(hWnd, "Was not able to save file.", "LSWallpaper", MB_OK | MB_ICONSTOP);
        }
        break;
      case IDC_CLOSE:
        if(wNotifyCode == BN_CLICKED) {
          // Close Wallpaper Manager
          // Ask to save changes
          SendMessage(hWnd, WM_CLOSE, 0, 0);
        }
        break;
      }
      return 0;
    }
    break;
  }

  return 0;
}

int ShowWallpaperManager(BOOL mod)
{
  modal = mod;

  if(!hDlg && !modal) {
    hDlg = CreateDialog(hInst, MAKEINTRESOURCE(IDD_WALLPAPERMANAGER), GetLitestepWnd(), WallpaperManagerDlgProc);
    ShowWindow(hDlg, SW_SHOWNORMAL);
  } else if(!hDlg && modal) {
    saved = FALSE;
    return DialogBox(hInst, MAKEINTRESOURCE(IDD_WALLPAPERMANAGER), GetLitestepWnd(), WallpaperManagerDlgProc);
  } else if(hDlg && !modal) {
    ShowWindow(hDlg, SW_SHOWNORMAL);
  }

  return 0;
}

void CloseAndDestroyManager()
{
  SendMessage(hDlg, WM_CLOSE, 0, 0);
}