/***********************************************************
*        LSWallpaper.c - Litestep Wallpaper Changer        *
* This program scans a list of bitmap files and, depending *
* upon options set by the user, changes the wallpaper at   *
* certain intervals                                        *
*                         *  *  *  *                       *
* Coding:                                                  *
*   Visigoth (Shaheen Gandhi in real life)                 *
*   e-mail: sgandhi@andrew.cmu.edu                         *
*                         *  *  *  *                       *
* Last Update:  October 20, 1999 7:30 AM                   *
*                         *  *  *  *                       *
* Copyright (c) 1999 Shaheen Gandhi                        *
***********************************************************/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <direct.h>
#include "lswallpaper.h"
#include "lsapi.h"

int numpapers = 0;
long curtime;
char *szApp = "LSWallpaper", *szModuleIniPath, image_path[512], last_image_path[512];
HINSTANCE hInst;
UINT idTimer;
struct Settings settings;
struct WallpaperRecordList *wallpapers, *current;


/***********************************************************
* void GetLastImagePath()                                  *
*                         *  *  *  *                       *
* Retrieves the path of the last used image as a wallpaper *
* stored in the Modules.ini file.  Retrieves "" if it is   *
* not available.                                           *
***********************************************************/

void GetLastImagePath()
{
  GetPrivateProfileString("WALLPAPER", "LastImagePath", "", image_path, 511, szModuleIniPath);
}


/***********************************************************
* void WriteLastImagePath()                                *
*                         *  *  *  *                       *
* Writes out the last image used by LSWallpaper.           *
***********************************************************/

void WriteLastImagePath()
{
  WritePrivateProfileString("WALLPAPER", "LastImagePath", image_path, szModuleIniPath);
  strcpy(last_image_path, image_path);
}


/***********************************************************
* struct Settings ReadSettings()                           *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - wharfDataType *wd                                   *
*      Pointer to Litestep's structure for general info.   *
*                         *  *  *  *                       *
* Retrieves all the settings in the RC file and stores     *
* it all in a local structure.                             *
***********************************************************/

struct Settings ReadSettings(wharfDataType *wd)
{
  struct Settings settings;
  szModuleIniPath = (char *)malloc(strlen(wd->lsPath) + strlen("\\MODULES.INI") + 1);
  strcpy(szModuleIniPath, wd->lsPath);
  strcat(szModuleIniPath, "\\MODULES.INI");
  GetRCString("PixmapPath", settings.BitmapPath, "", 511);
  GetRCString("WallpaperBitmapPath", settings.BitmapPath, settings.BitmapPath, 511);
  GetRCString("WallpaperTimeDelay", settings.TimeDelay, "00:00:00", 9);
  GetRCString("WallpaperListPath", settings.WallpaperListPath, "wallpapers.list", 511);
  settings.Sequenced = GetRCBool("WallpaperSequenced", TRUE);
  settings.ReplaceAtStartup = GetRCBool("WallpaperStartup", TRUE);
  settings.hours = atoi(strtok(settings.TimeDelay, ":"));
  settings.minutes = atoi(strtok(NULL, ":"));
  settings.seconds = atoi(strtok(NULL, ":"));
  settings.totaltime = (settings.hours*3600 + settings.minutes*60 + settings.seconds)*1000;
  GetLastImagePath();
  return settings;
}


/***********************************************************
* void WPRLMovetoHead()                                    *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*                         *  *  *  *                       *
* Moves the list pointer to the head of the list           *
***********************************************************/

void WPRLMovetoHead(struct WallpaperRecordList **listptr)
{
  if(*listptr)
    while((*listptr)->prev)
      *listptr = (struct WallpaperRecordList *)(*listptr)->prev;
}


/***********************************************************
* void WPRLMovetoTail()                                    *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*                         *  *  *  *                       *
* Moves the list pointer to the tail of the list           *
***********************************************************/

void WPRLMovetoTail(struct WallpaperRecordList **listptr)
{
  if(*listptr)
    while((*listptr)->next)
      *listptr = (struct WallpaperRecordList *)(*listptr)->next;
}


/***********************************************************
* void WPRLMovetoEntry()                                   *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*    - char *entry                                         *
*      Pointer to a path to search for                     *
*                         *  *  *  *                       *
* Searches the list for a particular entry and stops when  *
* it gets to the first one with a match                    *
***********************************************************/

BOOL WPRLMovetoEntry(struct WallpaperRecordList **listptr, char *entry)
{
  WPRLMovetoHead(listptr);
  if(*entry) {
    while(*listptr) {
      if(!_stricmp((*listptr)->wpr->path, entry))
        return TRUE;

      if((*listptr)->next)
        *listptr = (struct WallpaperRecordList *)(*listptr)->next;
      else
        break;
    }
  }

  return FALSE;
}


/***********************************************************
* void WPRLMoveNext()                                      *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*    - int i                                               *
*      The number of entries to move.                      *
*                         *  *  *  *                       *
* Moves i entries in the list.  Note: i may be negative.   *
* Note that we wrap around to the head or tail of the list *
* if we get to the end of it, but we still have to move    *
* down or up more entries.                                 *
***********************************************************/

void WPRLMoveNext(struct WallpaperRecordList **listptr, int i)
{
  int j;

  if(*listptr) {
    if(i >= 0)
      for(j = 1; j <= i; j++) {
        if((*listptr)->next)
          *listptr = (struct WallpaperRecordList *)(*listptr)->next;
        else {
          WPRLMovetoHead(listptr);
          break;
        }
      }
    else
      for(j = -1; j >= i; j--) {
        if((*listptr)->prev)
          *listptr = (struct WallpaperRecordList *)(*listptr)->prev;
        else {
          WPRLMovetoTail(listptr);
          break;
        }
      }

    if(j < i && i > 0)
      WPRLMoveNext(listptr, i - j);
    else if(j > i && i < 0)
      WPRLMoveNext(listptr, i - j);
  }
}


/***********************************************************
* struct WallpaperRecordList *WPRLAdd()                    *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*    - struct WallpaperRecord *data                        *
*      Pointer to a data describing the path and tiling of *
*      a wallpaper.                                        *
*                         *  *  *  *                       *
* Adds a wallpaper record to the list.  If there is no     *
* list, it creates one.                                    *
***********************************************************/

struct WallpaperRecordList *WPRLAdd(struct WallpaperRecordList **listptr, struct WallpaperRecord *data)
{
  if(*listptr) {
    WPRLMovetoTail(listptr);
    (*listptr)->next = (struct WallpaperRecordList *)malloc(sizeof(struct WallpaperRecordList));
    (*listptr)->next->next = NULL;
    (*listptr)->next->prev = (struct WallpaperRecordList *)*listptr;
    (*listptr)->next->wpr = data;
  } else {
    *listptr = (struct WallpaperRecordList *)malloc(sizeof(struct WallpaperRecordList));
    (*listptr)->next = NULL;
    (*listptr)->prev = NULL;
    (*listptr)->wpr = (struct WallpaperRecord *)data;
  }

  numpapers++;
  *listptr = ((*listptr)->next) ? (struct WallpaperRecordList *)(*listptr)->next : (struct WallpaperRecordList *)*listptr;

  return *listptr;
}


/***********************************************************
* struct WallpaperRecordList *WPRLFree()                   *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*                         *  *  *  *                       *
* Frees the memory used by one entry in the list.          *
***********************************************************/

struct WallpaperRecordList *WPRLFree(struct WallpaperRecordList **listptr)
{
  struct WallpaperRecordList *retval = NULL;

  if(*listptr) {
    if((*listptr)->wpr) {
      if((*listptr)->wpr->path)
        free((*listptr)->wpr->path);
      free((*listptr)->wpr);
    }
    
    if((*listptr)->prev) {
      (*listptr)->prev->next = (*listptr)->next;
      retval = (*listptr)->prev;
    }

    if((*listptr)->next) {
      (*listptr)->next->prev = (*listptr)->prev;
      retval = (*listptr)->next;
    }

    free(*listptr);
    numpapers--;
  }

  return retval;
}


/***********************************************************
* struct WallpaperRecordList *WPRLRemoveAll()              *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*                         *  *  *  *                       *
* Removes all entries from the list                        *
***********************************************************/

struct WallpaperRecordList *WPRLRemoveAll(struct WallpaperRecordList **listptr)
{
  while((*listptr)->next)
    *listptr = WPRLFree(listptr);
  WPRLFree(listptr);

    /*
    if(!(*listptr)->prev) {
      temp = (*listptr)->next;
      WPRLFree(listptr);
      *listptr = temp;
      if(*listptr)
        (*listptr)->prev = NULL;
    } else {
      if(!(*listptr)->next) {
        temp = (struct WallpaperRecordList *)(*listptr)->prev;
        WPRLFree(listptr);
        *listptr = (struct WallpaperRecordList *)temp;
        (*listptr)->next = NULL;
      } else {
        temp = (struct WallpaperRecordList *)(*listptr)->prev;
        temp2 = (struct WallpaperRecordList *)(*listptr)->next;
        WPRLFree(listptr);
        temp->next = (struct WallpaperRecordList *)temp2;
        temp2->prev = (struct WallpaperRecordList *)temp;
        *listptr = temp;
      }
    }
  }
  */

  return *listptr;
}


/***********************************************************
* int ParseWallpapers()                                    *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct Settings *settings                           *
*      Pointer to a structure containing all the step.rc   *
*      settings                                            *
*                         *  *  *  *                       *
* Retrieves all the wallpapers listed in the               *
* wallpapers.list file and builds up records for them.     *
* Returns the number of wallpapers parsed, or -1 upon      *
* error.
***********************************************************/

int ParseWallpapers(struct Settings *settings)
{
  BOOL tile;
  char listline[512], *path, *pointer;
  FILE *fpPaperList;
  struct WallpaperRecord *wpr;

  fpPaperList = fopen(settings->WallpaperListPath, "r");
  if(!fpPaperList) {
    if(MessageBox(NULL, "You have not specified a wallpaper list, or it does not exist.  Would you like to use the Wallpaper Manager to create one now?", "LSWallpaper", MB_YESNO) == IDYES) {
      if(ShowWallpaperManager(TRUE)) {
        fpPaperList = fopen(settings->WallpaperListPath, "r");
        if(!fpPaperList)
          return -1;
      } else
        return -1;
    } else
      return -1;
  }

  for(;;) {
    if(!fgets(listline, 511, fpPaperList))
      break;

    pointer = strstr(listline, "//");
    if(pointer == listline)
      continue;

    if(pointer)
      *pointer = 0;

    path = strtok(listline, "\t");
    pointer = strtok(NULL, "\t ");
    if(pointer)
      tile = atoi(pointer);
    else
      tile = 0;

    if(path) {
      pointer = strchr(path, '\\');
      wpr = (struct WallpaperRecord *)malloc(sizeof(struct WallpaperRecord));
      wpr->path = pointer ? (char *)malloc(strlen(path) + 1) : (char *)malloc(strlen(settings->BitmapPath) +strlen(path) + 2);
      wpr->tiled = tile;

      if(pointer)
        strcpy(wpr->path, path);
      else {
        pointer = settings->BitmapPath + strlen(settings->BitmapPath) - 1;
        strcpy(wpr->path, settings->BitmapPath);
        if(!(strrchr(settings->BitmapPath, '\\') == pointer))
          strcat(wpr->path, "\\");

        strcat(wpr->path, path);
      }

      current = WPRLAdd(&wallpapers, (struct WallpaperRecord *)wpr);
    } 
  }

  fclose(fpPaperList);
  WPRLMovetoHead(&wallpapers);

  return numpapers;
}


/***********************************************************
* BOOL SetTiledRegKey()                                    *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList *listptr                 *
*      Pointer to a list element                           *
*                         *  *  *  *                       *
* Sets the Registry Key which tells Windows whether to     *
* Tile the wallpaper.  Returns TRUE if successful.         *
***********************************************************/

BOOL SetTiledRegKey(struct WallpaperRecordList *listptr)
{
  HKEY hKey;
  char szValue[17];

  _itoa(listptr->wpr->tiled, szValue, 10);
  
  if(RegOpenKeyEx(HKEY_CURRENT_USER, "Control Panel\\Desktop", 0, KEY_WRITE, &hKey) == ERROR_SUCCESS) {
    if(!(RegSetValueEx(hKey, "TileWallpaper", 0, REG_SZ, (unsigned char *)szValue, strlen(szValue) + 1)) == ERROR_SUCCESS)
      return FALSE;
    else {
      RegCloseKey(hKey);
      return TRUE;
    }
  }
  else
    return FALSE;
}


/***********************************************************
* int UseNextWallpaper()                                   *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*                         *  *  *  *                       *
* Uses the next wallpaper in the list of wallpapers.       *
* Returns -1 on error or SystemParametersInfo on success.  *
***********************************************************/

int UseNextWallpaper(struct WallpaperRecordList **listptr)
{
  if(*listptr) {
    strcpy(last_image_path, image_path);
    WPRLMoveNext(listptr, 1);
    strcpy(image_path, (*listptr)->wpr->path);
    if(SetTiledRegKey(*listptr)) {
      if(SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, image_path, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE))
        WriteLastImagePath();
      else return 0;
      return 1;
    } else
      return -1;
  } else
    return -1;
}


/***********************************************************
* int UsePreviousWallpaper()                               *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*                         *  *  *  *                       *
* Uses the previous wallpaper in the list of wallpapers.   *
* Returns -1 on error or SystemParametersInfo on success.  *
***********************************************************/

int UsePreviousWallpaper(struct WallpaperRecordList **listptr)
{
  char *old_path = (char *)malloc(strlen(last_image_path) + 1);

  if(*listptr) {
    WPRLMovetoEntry(listptr, last_image_path);
    strcpy(old_path, last_image_path);
    strcpy(last_image_path, image_path);
    strcpy(image_path, old_path);
    free(old_path);
    if(SetTiledRegKey(*listptr)) {
      if(SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, last_image_path, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE))
        WriteLastImagePath();
      else return 0;
      return 1;
    } else
      return -1;
  } else
    return -1;
}


/***********************************************************
* int UseSpecificWallpaper()                               *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*    - char *bitmap                                        *
*      Path of the bitmap to move to in the list.          *
*                         *  *  *  *                       *
* Uses the previous wallpaper in the list of wallpapers.   *
* Returns -1 on error or SystemParametersInfo on success.  *
***********************************************************/

int UseSpecificWallpaper(struct WallpaperRecordList **listptr, char *bitmap)
{
  char *full_path, *p;

  p = strrchr(bitmap, '\\');
  if(p) {
    full_path = bitmap;
  } else {
    full_path = (char *)malloc(strlen(settings.BitmapPath) + strlen(bitmap) + 2);
    p = settings.BitmapPath + strlen(settings.BitmapPath) - 1;
    strcpy(full_path, settings.BitmapPath);
    if(!(strrchr(settings.BitmapPath, '\\') == p))
      strcat(full_path, "\\");
    strcat(full_path, bitmap);
  }

  if(!WPRLMovetoEntry(listptr, full_path))
    return -1;

  strcpy(last_image_path, image_path);
  strcpy(image_path, (*listptr)->wpr->path);
  if(SetTiledRegKey(*listptr)) {
    if(SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, image_path, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE))
      WriteLastImagePath();
    else return 0;
    return 1;
  }
  else
    return -2;

}


/***********************************************************
* int UseRandomWallpaper()                                 *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*                         *  *  *  *                       *
* Uses a random wallpaper in the list of wallpapers.       *
* Returns -1 on error or SystemParametersInfo on success.  *
***********************************************************/

int UseRandomWallpaper(struct WallpaperRecordList **listptr)
{
  int r;

  if(*listptr) {
    r = rand() % numpapers;
    WPRLMovetoHead(listptr);
    WPRLMoveNext(listptr, r);
    strcpy(last_image_path, image_path);
    strcpy(image_path, (*listptr)->wpr->path);
    if(SetTiledRegKey(*listptr)) {
      if(SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, image_path, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE))
        WriteLastImagePath();
      else
        return 0;
      return 1;
    } else
      return -1;
  } else
    return -1;
}


/***********************************************************
* int ChangeWallpaper()                                    *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - struct WallpaperRecordList **listptr                *
*      Pointer to a pointer to an element of the list of   *
*      wallpaper records.                                  *
*    - struct Settings *settings                           *
*      Pointer to a structure containing all step.rc       *
*      settings.                                           *
*                         *  *  *  *                       *
* Changes the wallpaper depending upon whether the user    *
* has selected Sequenced or Random Selection.              *
***********************************************************/

int ChangeWallpaper(struct WallpaperRecordList **listptr, struct Settings *settings)
{

  if(listptr) {
    switch(settings->Sequenced) {
    case TRUE:
      return UseNextWallpaper(listptr);
      break;

    case FALSE:
      return UseRandomWallpaper(listptr);
      break;

    default:
      return -1;
      break;
    }
  }

  return -1;
}


/***********************************************************
* VOID CALLBACK TimerProc()                                *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HWND hwnd                                           *
*      The HWND of the window that owns this timer         *
*    - UINT wMsg                                           *
*      The message sent by Windows that triggers the timer *
*    - UINT idEvent                                        *
*      Some type of identifier for the timer.              *
*    - DWORD dwTime                                        *
*      Time elapsed since Windows booted, I think          *
*                         *  *  *  *                       *
* Handles the timer function to change the wallpaper       *
***********************************************************/

VOID CALLBACK TimerProc(HWND hwnd, UINT wMsg, UINT idEvent, DWORD dwTime)
{
  ChangeWallpaper(&current, &settings);
  /*curtime = settings.totaltime/1000;*/
}

/*VOID CALLBACK Counter(HWND hwnd, UINT wMsg, UINT idEvent, DWORD dwTime)
{
  curtime--;
}*/


/***********************************************************
* void BangNextWallpaper()                                 *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HWND caller                                         *
*      The HWND of the window that called this !Bang       *
*      Command.                                            *
*    - char *args                                          *
*      Should be the arguments to the function...          *
*                         *  *  *  *                       *
* Handles the !Bang Command to move to the next wallpaper. *
***********************************************************/

void BangNextWallpaper(HWND caller, char *args)
{
  TIMERPROC lpfnTimer = (TIMERPROC)TimerProc;
  if(idTimer)
    KillTimer(NULL, idTimer);

  UseNextWallpaper(&current);

  if(idTimer)
    idTimer = SetTimer(NULL, WM_TIMER, settings.totaltime, lpfnTimer);
}


/***********************************************************
* void BangPreviousWallpaper()                             *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HWND caller                                         *
*      The HWND of the window that called this !Bang       *
*      Command.                                            *
*    - char *args                                          *
*      Should be the arguments to the function...          *
*                         *  *  *  *                       *
* Handles the !Bang Command to move to the previoius       *
* wallpaper.                                               *
***********************************************************/

void BangPreviousWallpaper(HWND caller, char *args)
{
  TIMERPROC lpfnTimer = (TIMERPROC)TimerProc;
  if(idTimer)
    KillTimer(NULL, idTimer);

  UsePreviousWallpaper(&current);

  if(idTimer)
    idTimer = SetTimer(NULL, WM_TIMER, settings.totaltime, lpfnTimer);
}


/***********************************************************
* void BangChangeWallpaper()                               *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HWND caller                                         *
*      The HWND of the window that called this !Bang       *
*      Command.                                            *
*    - char *args                                          *
*      Should be the arguments to the function...          *
*                         *  *  *  *                       *
* Handles the !Bang Command to rescan the wallpaper list.  *
***********************************************************/

void BangChangeWallpaper(HWND caller, char *args)
{
  TIMERPROC lpfnTimer = (TIMERPROC)TimerProc;
  if(idTimer)
    KillTimer(NULL, idTimer);

  if(*args == '\0')
    ChangeWallpaper(&current, &settings);
  else
    if(UseSpecificWallpaper(&current, args) == -1)
      MessageBox(NULL, "Could not find file.", szApp, MB_OK);

  if(idTimer)
    idTimer = SetTimer(NULL, WM_TIMER, settings.totaltime, lpfnTimer);
}


/***********************************************************
* void BangRescanWallpaperList()                           *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HWND caller                                         *
*      The HWND of the window that called this !Bang       *
*      Command.                                            *
*    - char *args                                          *
*      Should be the arguments to the function...          *
*                         *  *  *  *                       *
* Handles the !Bang Command to rescan the wallpaper list.  *
***********************************************************/

void BangRescanWallpaperList(HWND caller, char *args)
{
  WPRLRemoveAll(&wallpapers);
  ParseWallpapers(&settings);
}


/***********************************************************
* void BangAboutLSWallpaper()                              *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HWND caller                                         *
*      The HWND of the window that called this !Bang       *
*      Command.                                            *
*    - char *args                                          *
*      Should be the arguments to the function...          *
*                         *  *  *  *                       *
* Handles the !Bang Command to tell the user about         *
* LSWallpaper.                                             *
***********************************************************/

void BangAboutLSWallpaper(HWND caller, char *args)
{
  MessageBox(NULL, "LSWallpaper v1.0 by Visigoth (aka Shaheen Gandhi)\n\nSend bug reports to: gandhimail@worldnet.att.net", "About LSWallpaper", MB_OK);
}


/***********************************************************
* void BangManageWallpapers()                              *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HWND caller                                         *
*      The HWND of the window that called this !Bang       *
*      Command.                                            *
*    - char *args                                          *
*      Should be the arguments to the function...          *
*                         *  *  *  *                       *
* Brings up the Wallpaper Manager                          *
***********************************************************/

void BangManageWallpapers(HWND caller, char *args)
{
  ShowWallpaperManager(FALSE);
}


/*void BangWallpaperTime(HWND caller, char *args)
{
  char msg[256];
  long timeleft = curtime/1000;
  int hours, minutes, seconds;

  hours = timeleft/3600;
  minutes = timeleft/60 - settings.hours*60;
  seconds = timeleft - settings.hours*3600 - settings.minutes*60;
  sprintf(msg, "The time left for a wallpaper change is:  %d:%d:%d", hours, minutes, seconds);
  MessageBox(NULL, msg, "LSWallpaper", MB_OK);
}*/


/***********************************************************
* int initModule()                                         *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HWND parent                                         *
*      The HWND of the parent window to this module        *
*    - HINSTANCE dll                                       *
*      The HINSTANCE of this DLL.                          *
*    - WharfDataType *wd                                   *
*      A pointer to the structure containing data about LS *
*                         *  *  *  *                       *
* Handles all initialization of LSWallpaper.  Returns 0    *
***********************************************************/

int initModule(HWND parent, HINSTANCE dll, wharfDataType *wd)
{
  TIMERPROC lpfnTimer = (TIMERPROC)TimerProc;
  /*TIMERPROC lpfnCounter = (TIMERPROC)Counter;*/
  hInst = dll;
  settings = ReadSettings(wd);
  wallpapers = NULL;

  srand((unsigned)time(NULL));
  AddBangCommand("!NEXTWALLPAPER", BangNextWallpaper);
  AddBangCommand("!PREVIOUSWALLPAPER", BangPreviousWallpaper);
  AddBangCommand("!ABOUTLSWALLPAPER", BangAboutLSWallpaper);
  AddBangCommand("!RESCANWALLPAPERLIST", BangRescanWallpaperList);
  AddBangCommand("!CHANGEWALLPAPER", BangChangeWallpaper);
  AddBangCommand("!WALLPAPERMANAGER", BangManageWallpapers);
  /*AddBangCommand("!WALLPAPERTIME", BangWallpaperTime);*/
  if(ParseWallpapers(&settings) == -1)
    return 0;

  if(!WPRLMovetoEntry(&current, image_path))
    WPRLMovetoTail(&current);

  if(settings.ReplaceAtStartup && (!(GetAsyncKeyState(VK_SHIFT) & 0x8000)))
    ChangeWallpaper(&current, &settings);

  if(settings.totaltime) {
    idTimer = SetTimer(NULL, WM_TIMER, settings.totaltime, lpfnTimer);
  
    if(!idTimer)
      MessageBox(NULL, "Unable to allocate timer.", "LSWallpaper", MB_OK);
  }

  return 0;
}


/***********************************************************
* int quitModule()                                         *
*                         *  *  *  *                       *
*    Arguments:                                            *
*                                                          *
*    - HINSTANCE dll                                       *
*      The HINSTANCE of this DLL.                          *
*                         *  *  *  *                       *
* Handles all of the unloading of this DLL.                *
***********************************************************/

int quitModule(HINSTANCE dll)
{
  // ADD: CHECK FOR MANAGER
  // Did the user change stuff in the manager
  // and didn't save the file?

  CloseAndDestroyManager();
  WPRLRemoveAll(&current);
  WriteLastImagePath();

  if(idTimer)
    KillTimer(NULL, idTimer);

  return 0;
}