// shellext.cpp : Implementation of DLL Exports.
//
// Author: Glenn Carr (GlennCarr@email.com)
// Please send any bugs or fixes to GlennCarr@email.com.
//

#include "stdafx.h"
#include "resource.h"
#include <initguid.h>
#include "lswallpaperext.h"

#include "lswallpaperext_i.c"
#include "shellext.h"

CComModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
OBJECT_ENTRY(CLSID_LSWallpaperExt, CLSWallpaperExt)
END_OBJECT_MAP()

/////////////////////////////////////////////////////////////////////////////
// DLL Entry Point

BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        _Module.Init(ObjectMap, hInstance, &LIBID_LSWallpaperLib);
        DisableThreadLibraryCalls(hInstance);
    }
    else if (dwReason == DLL_PROCESS_DETACH)
        _Module.Term();
    return TRUE;    // ok
}

/////////////////////////////////////////////////////////////////////////////
// Used to determine whether the DLL can be unloaded by OLE

STDAPI DllCanUnloadNow(void)
{
    return (_Module.GetLockCount()==0) ? S_OK : S_FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Returns a class factory to create an object of the requested type

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
    return _Module.GetClassObject(rclsid, riid, ppv);
}

/////////////////////////////////////////////////////////////////////////////
// DllRegisterServer - Adds entries to the system registry

STDAPI DllRegisterServer(void)
{
    // registers object, typelib and all interfaces in typelib
    return _Module.RegisterServer(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// DllUnregisterServer - Removes entries from the system registry

STDAPI DllUnregisterServer(void)
{
    return _Module.UnregisterServer(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// CLSWallpaperExt - Extension Class

HRESULT CLSWallpaperExt::QueryContextMenu(HMENU hMenu, UINT indexMenu, UINT idCmd, UINT, UINT)
{
  MENUITEMINFO item;
  char buf[12];
  item.cbSize = sizeof(MENUITEMINFO);
  item.dwTypeData = buf;
  item.cch = sizeof(buf);
  item.fMask = MIIM_SUBMENU | MIIM_TYPE;
  item.hSubMenu = lswpMenu;
  item.fType = MFT_STRING;

  ::InsertMenuItem(hMenu, indexMenu, true, &item);

  return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, 1);
}

HRESULT CLSWallpaperExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpcmCmdInfo)
{

  return S_OK;
}

HRESULT CLSWallpaperExt::GetCommandString(UINT idCmd, UINT uFlags, UINT *, LPSTR szText, UINT dwMaxChars)
{
  memset(szText, 0, dwMaxChars);

  if(idCmd = IDM_ADDWALLPAPERS)
    if(m_cFiles == 1)
      _snprintf(szText, dwMaxChars - 1, "Add %s to the list of wallpapers", m_szFile);
    else
      strncpy(szText, "Adds selected files to the list of wallpapers", dwMaxChars - 1);
  else
    strncpy(szText, "Manage Wallpaper List", dwMaxChars - 1);

  return S_OK;
}

HRESULT CLSWallpaperExt::Initialize(LPCITEMIDLIST /*pidlFolder*/, LPDATAOBJECT lpdobj, HKEY /*hKeyProgID*/)
{
	if ( lpdobj == NULL )
		return E_INVALIDARG;

	// Get the data as CF_HDROP
	STGMEDIUM medium;
	FORMATETC fe = { CF_HDROP, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
	HRESULT hr = lpdobj->GetData( &fe, &medium );
	if ( FAILED( hr ) )
		return E_INVALIDARG;

	// Get the number of selected files
	m_cFiles = DragQueryFile( reinterpret_cast<HDROP>(medium.hGlobal), 0xFFFFFFFF, NULL, NULL );
	
	// Build the list of file names
	m_lstFiles.clear();
	for ( int iFile = 0; iFile < m_cFiles; iFile++ )
	{
		// Get the name of the selected file
		memset( m_szFile, 0, sizeof m_szFile );
		DragQueryFile( reinterpret_cast<HDROP>(medium.hGlobal), 
						 iFile, 
						 m_szFile, 
						 sizeof(m_szFile)/sizeof(m_szFile[0]) );
		m_lstFiles.push_back( m_szFile );
	}

	ReleaseStgMedium(&medium);
	return hr;
};